@@ -2,14 +2,15 @@
    Copyright (C) 1997-9 Eddie Kohler, eddietwo@lcs.mit.edu
    This file is part of the GIF library.
 
-   The GIF library is free software*. It is distributed under the GNU Public
-   License, version 2 or later; you can copy, distribute, or alter it at will,
-   as long as this notice is kept intact and this source code is made
+   The GIF library is free software*. It is distributed under the GNU General
+   Public License, version 2 or later; you can copy, distribute, or alter it
+   at will, as long as this notice is kept intact and this source code is made
    available. There is no warranty, express or implied.
 
-   *The LZW compression method used by GIFs is patented. Unisys, the patent
-   holder, allows the compression algorithm to be used without a license in
-   software distributed at no cost to the user. */
+   *There is a patent on the LZW compression method used by GIFs, and included
+   in gifwrite.c. Unisys, the patent holder, allows the compression algorithm
+   to be used without a license in software distributed at no cost to the
+   user. The decompression algorithm is not patented. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -31,10 +32,10 @@ typedef struct {
   
   Gif_Code *prefix;
   byte *suffix;
-  u_int16_t *length;
+  uint16_t *length;
   
-  u_int16_t width;
-  u_int16_t height;
+  uint16_t width;
+  uint16_t height;
   
   byte *image;
   byte *maximage;
@@ -51,11 +52,13 @@ typedef struct Gif_Reader {
   
   FILE *f;
   const byte *v;
-  u_int32_t w;
+  uint32_t w;
+  uint32_t length;
   int is_record;
   int is_eoi;
   byte (*byte_getter)(struct Gif_Reader *);
-  void (*block_getter)(byte *, u_int32_t, struct Gif_Reader *);
+  void (*block_getter)(byte *, uint32_t, struct Gif_Reader *);
+  uint32_t (*offseter)(struct Gif_Reader *);
   int (*eofer)(struct Gif_Reader *);
   
 } Gif_Reader;
@@ -64,9 +67,10 @@ typedef struct Gif_Reader {
 #define gifgetc(grr)	((char)(*grr->byte_getter)(grr))
 #define gifgetbyte(grr) ((*grr->byte_getter)(grr))
 #define gifgetblock(ptr, size, grr) ((*grr->block_getter)(ptr, size, grr))
+#define gifgetoffset(grr) ((*grr->offseter)(grr))
 #define gifeof(grr)	((*grr->eofer)(grr))
 
-static inline u_int16_t
+static inline uint16_t
 gifgetunsigned(Gif_Reader *grr)
 {
   byte one = gifgetbyte(grr);
@@ -83,15 +87,27 @@ file_byte_getter(Gif_Reader *grr)
 }
 
 static void
-file_block_getter(byte *p, u_int32_t s, Gif_Reader *grr)
+file_block_getter(byte *p, uint32_t s, Gif_Reader *grr)
 {
-  fread(p, s, 1, grr->f);
+  fread(p, 1, s, grr->f);
+}
+
+static uint32_t
+file_offseter(Gif_Reader *grr)
+{
+  return ftell(grr->f);
 }
 
 static int
 file_eofer(Gif_Reader *grr)
 {
-  return feof(grr->f);
+  int c = getc(grr->f);
+  if (c == EOF)
+    return 1;
+  else {
+    ungetc(c, grr->f);
+    return 0;
+  }
 }
 
 
@@ -102,13 +118,19 @@ record_byte_getter(Gif_Reader *grr)
 }
 
 static void
-record_block_getter(byte *p, u_int32_t s, Gif_Reader *grr)
+record_block_getter(byte *p, uint32_t s, Gif_Reader *grr)
 {
   if (s > grr->w) s = grr->w;
   memcpy(p, grr->v, s);
   grr->w -= s, grr->v += s;
 }
 
+static uint32_t
+record_offseter(Gif_Reader *grr)
+{
+  return grr->length - grr->w;
+}
+
 static int
 record_eofer(Gif_Reader *grr)
 {
@@ -117,13 +139,15 @@ record_eofer(Gif_Reader *grr)
 
 
 static void
-make_data_reader(Gif_Reader *grr, const byte *data, u_int32_t length)
+make_data_reader(Gif_Reader *grr, const byte *data, uint32_t length)
 {
   grr->v = data;
+  grr->length = length;
   grr->w = length;
   grr->is_record = 1;
   grr->byte_getter = record_byte_getter;
   grr->block_getter = record_block_getter;
+  grr->offseter = record_offseter;
   grr->eofer = record_eofer;
 }
 
@@ -144,7 +168,7 @@ one_code(Gif_Context *gfc, Gif_Code code)
   Gif_Code *prefixes = gfc->prefix;
   byte *ptr;
   int lastsuffix;
-  u_int16_t codelength = gfc->length[code];
+  uint16_t codelength = gfc->length[code];
 
   gfc->decodepos += codelength;
   ptr = gfc->image + gfc->decodepos;
@@ -189,7 +213,7 @@ read_image_block(Gif_Reader *grr, byte *buffer, int *bit_pos_store,
       bit_length -= i * 8;
     }
     block_len = gifgetbyte(grr);
-    GIF_DEBUG(("image_block(%d)", block_len));
+    GIF_DEBUG(("\nimage_block(%d)", block_len));
     if (block_len == 0) return 0;
     gifgetblock(buffer + bit_length / 8, block_len, grr);
     bit_length += block_len * 8;
@@ -208,7 +232,7 @@ read_image_data(Gif_Context *gfc, Gif_Reader *grr)
      across blocks */
   byte buffer[GIF_MAX_BLOCK + 5];
   int i;
-  u_int32_t accum;
+  uint32_t accum;
   
   int bit_position;
   int bit_length;
@@ -227,7 +251,7 @@ read_image_data(Gif_Context *gfc, Gif_Reader *grr)
   gfc->decodepos = 0;
   
   min_code_size = gifgetbyte(grr);
-  GIF_DEBUG(("min_code_size(%d)", min_code_size));
+  GIF_DEBUG(("\n\nmin_code_size(%d)", min_code_size));
   if (min_code_size >= GIF_MAX_CODE_BITS) {
     gif_read_error(gfc, "min_code_size too big");
     min_code_size = GIF_MAX_CODE_BITS - 1;
@@ -278,7 +302,7 @@ read_image_data(Gif_Context *gfc, Gif_Reader *grr)
     code = (Gif_Code)((accum >> (bit_position % 8)) & CUR_CODE_MASK);
     bit_position += bits_needed;
     
-    GIF_DEBUG(("code(%d)", code));
+    GIF_DEBUG(("%d", code));
     
     /* CHECK FOR SPECIAL OR BAD CODES: clear_code, eoi_code, or a code that is
      * too large. */
@@ -336,11 +360,11 @@ read_image_data(Gif_Context *gfc, Gif_Reader *grr)
   
   /* read blocks until zero-length reached. */
   i = gifgetbyte(grr);
-  GIF_DEBUG(("after_image(%d)", i));
+  GIF_DEBUG(("\nafter_image(%d)\n", i));
   while (i > 0) {
     gifgetblock(buffer, i, grr);
     i = gifgetbyte(grr);
-    GIF_DEBUG(("after_image(%d)", i));
+    GIF_DEBUG(("\nafter_image(%d)\n", i));
   }
   
   /* zero-length block reached. */
@@ -404,7 +428,7 @@ read_compressed_image(Gif_Image *gfi, Gif_Reader *grr, int read_flags)
 {
   if (grr->is_record) {
     const byte *first = grr->v;
-    u_int32_t pos;
+    uint32_t pos;
     
     /* scan over image */
     pos = 1;			/* skip min code size */
@@ -432,8 +456,8 @@ read_compressed_image(Gif_Image *gfi, Gif_Reader *grr, int read_flags)
     
   } else {
     /* non-record; have to read it block by block. */
-    u_int32_t comp_cap = 1024;
-    u_int32_t comp_len;
+    uint32_t comp_cap = 1024;
+    uint32_t comp_len;
     byte *comp = Gif_NewArray(byte, comp_cap);
     int i;
     if (!comp) return 0;
@@ -502,7 +526,7 @@ Gif_FullUncompressImage(Gif_Image *gfi, Gif_ReadErrorHandler h, void *hthunk)
   gfc.stream = &fake_gfs;
   gfc.prefix = Gif_NewArray(Gif_Code, GIF_MAX_CODE);
   gfc.suffix = Gif_NewArray(byte, GIF_MAX_CODE);
-  gfc.length = Gif_NewArray(u_int16_t, GIF_MAX_CODE);
+  gfc.length = Gif_NewArray(uint16_t, GIF_MAX_CODE);
   gfc.handler = h;
   gfc.handler_thunk = hthunk;
   
@@ -666,8 +690,7 @@ read_unknown_extension(Gif_Stream *gfs, int kind, char *app_name, int position,
 
 
 static int
-read_application_extension(Gif_Context *gfc, Gif_Image *gfi, int position,
-			   Gif_Reader *grr)
+read_application_extension(Gif_Context *gfc, int position, Gif_Reader *grr)
 {
   Gif_Stream *gfs = gfc->stream;
   byte buffer[GIF_MAX_BLOCK + 1];
@@ -724,7 +747,7 @@ read_gif(Gif_Reader *grr, int read_flags,
   Gif_Image *new_gfi;
   Gif_Context gfc;
   int extension_position = 0;
-  int ok = 0;
+  int unknown_block_type = 0;
   
   if (gifgetc(grr) != 'G' ||
       gifgetc(grr) != 'I' ||
@@ -740,7 +763,7 @@ read_gif(Gif_Reader *grr, int read_flags,
   gfc.stream = gfs;
   gfc.prefix = Gif_NewArray(Gif_Code, GIF_MAX_CODE);
   gfc.suffix = Gif_NewArray(byte, GIF_MAX_CODE);
-  gfc.length = Gif_NewArray(u_int16_t, GIF_MAX_CODE);
+  gfc.length = Gif_NewArray(uint16_t, GIF_MAX_CODE);
   gfc.handler = handler;
   gfc.handler_thunk = handler_thunk;
   
@@ -759,7 +782,7 @@ read_gif(Gif_Reader *grr, int read_flags,
     switch (block) {
       
      case ',': /* image block */
-      GIF_DEBUG(("imageread"));
+      GIF_DEBUG(("imageread %d", gfs->nimages));
       
       gfi->identifier = last_name;
       last_name = 0;
@@ -772,18 +795,12 @@ read_gif(Gif_Reader *grr, int read_flags,
       new_gfi = Gif_NewImage();
       if (!new_gfi) goto done;
       
-      if (read_flags & GIF_READ_NETSCAPE_WORKAROUND) {
-	new_gfi->transparent = gfi->transparent;
-	new_gfi->delay = gfi->delay;
-	new_gfi->disposal = gfi->disposal;
-      }
       gfi = new_gfi;
       extension_position++;
       break;
       
      case ';': /* terminator */
-      GIF_DEBUG(("term"));
-      ok = 1;
+      GIF_DEBUG(("term\n"));
       goto done;
       
      case '!': /* extension */
@@ -804,7 +821,7 @@ read_gif(Gif_Reader *grr, int read_flags,
 	break;
 	
        case 0xFF:
-	read_application_extension(&gfc, gfi, extension_position, grr);
+	read_application_extension(&gfc, extension_position, grr);
 	break;
 	
        default:
@@ -815,8 +832,13 @@ read_gif(Gif_Reader *grr, int read_flags,
       break;
       
      default:
-      gif_read_error(&gfc, "unknown block type");
-      break;
+       if (!unknown_block_type) {
+	 char buf[256];
+	 sprintf(buf, "unknown block type %d at file offset %d", block, gifgetoffset(grr) - 1);
+	 gif_read_error(&gfc, buf);
+	 unknown_block_type = 1;
+       }
+       break;
       
     }
     
@@ -833,12 +855,14 @@ read_gif(Gif_Reader *grr, int read_flags,
   Gif_DeleteArray(gfc.prefix);
   Gif_DeleteArray(gfc.suffix);
   Gif_DeleteArray(gfc.length);
-  if (ok)
-    return gfs;
-  else {
-    Gif_DeleteStream(gfs);
-    return 0;
+
+  if (gfs->errors == 0 && !(read_flags & GIF_READ_TRAILING_GARBAGE_OK) && !grr->eofer(grr)) {
+    gif_read_error(&gfc, "trailing garbage after GIF ignored");
+    /* but clear error count, since the GIF itself was all right */
+    gfs->errors = 0;
   }
+  
+  return gfs;
 }
 
 
@@ -852,6 +876,7 @@ Gif_FullReadFile(FILE *f, int read_flags,
   grr.is_record = 0;
   grr.byte_getter = file_byte_getter;
   grr.block_getter = file_block_getter;
+  grr.offseter = file_offseter;
   grr.eofer = file_eofer;
   return read_gif(&grr, read_flags, h, hthunk);
 }
@@ -869,6 +894,22 @@ Gif_FullReadRecord(const Gif_Record *gifrec, int read_flags,
 }
 
 
+#undef Gif_ReadFile
+#undef Gif_ReadRecord
+
+Gif_Stream *
+Gif_ReadFile(FILE *f)
+{
+  return Gif_FullReadFile(f, GIF_READ_UNCOMPRESSED, 0, 0);
+}
+
+Gif_Stream *
+Gif_ReadRecord(const Gif_Record *gifrec)
+{
+  return Gif_FullReadRecord(gifrec, GIF_READ_UNCOMPRESSED, 0, 0);
+}
+
+
 #ifdef __cplusplus
 }
 #endif
