@@ -518,15 +518,15 @@ int
 main(int argc, char **argv)
 {
   // First, parse command-line options.
-  char *display_name = 0;
-  char *layout_name = 0;
-  char *tileset_name = "thick";
-  char *background_name = "default";
-  char *x_name = 0;
-  char *geometry = 0;
-  char *config_dir = PKGDATADIR;
+  const char *display_name = 0;
+  const char *layout_name = 0;
+  const char *tileset_name = "thick";
+  const char *background_name = "default";
+  const char *x_name = 0;
+  const char *geometry = 0;
+  const char *config_dir = PKGDATADIR;
   bool board_number_given = false;
-  u_int32_t board_number = 0;
+  uint32_t board_number = 0;
   bool do_config_list = false;
   
   Clp_Parser *clp =
@@ -700,11 +700,11 @@ particular purpose.\n");
     XClassHint class_hint;
     char *woog[2];
     XTextProperty window_name_prop, icon_name_prop;
-    woog[0] = x_name ? x_name : (char *)program_name;
+    woog[0] = (char *)(x_name ? x_name : program_name);
     woog[1] = NULL;
     XStringListToTextProperty(woog, 1, &window_name_prop);
     XStringListToTextProperty(woog, 1, &icon_name_prop);
-    class_hint.res_name = x_name ? x_name : (char *)program_name;
+    class_hint.res_name = (char *)(x_name ? x_name : program_name);
     class_hint.res_class = "XMahjongg";
 
     XResizeWindow(display, window, size_hint->width, size_hint->height);
