@@ -60,7 +60,7 @@ SolvableMaker::block_row(Tile *t, bool move_left)
 
 
 int
-SolvableMaker::blank_distance(Tile *t, bool move_left, bool first = true) const
+SolvableMaker::blank_distance(Tile *t, bool move_left, bool first) const
 {
   if (!t->real())
     return t == _null_tile ? -1 : 0;
@@ -222,7 +222,7 @@ SolvableMaker::try_assign(int *permuted_tiles)
 
 
 bool
-SolvableMaker::assign(u_int32_t seed)
+SolvableMaker::assign(uint32_t seed)
 {
   /* assign creates a board which is guaranteed to be solvable. I
      thought for a long time about how to do this but impotently! I was
